'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _command = require('@oclif/command');

var _liskElements = require('lisk-elements');

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _error = require('../../utils/error');

var _input = require('../../utils/input');

var _input2 = _interopRequireDefault(_input);

var _flags = require('../../utils/flags');

var _flags2 = _interopRequireDefault(_flags);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * LiskHQ/lisk-commander
 * Copyright © 2017–2018 Lisk Foundation
 *
 * See the LICENSE file at the top-level directory of this distribution
 * for licensing information.
 *
 * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
 * no part of this software, including this file, may be copied, modified,
 * propagated, or distributed except according to the terms contained in the
 * LICENSE file.
 *
 * Removal or modification of this copyright notice is prohibited.
 *
 */
const processInputs = (publicKey, signature, message) => ({ data }) => ({
	verified: _liskElements.cryptography.verifyMessageWithPublicKey({
		publicKey,
		signature,
		message: message || data
	})
});

class VerifyCommand extends _base2.default {
	async run() {
		const {
			args: { publicKey, signature, message },
			flags: { message: messageSource }
		} = this.parse(VerifyCommand);

		if (!message && !messageSource) {
			throw new _error.ValidationError('No message was provided.');
		}

		const inputs = await (0, _input2.default)({
			data: message ? null : { source: messageSource }
		});
		const result = processInputs(publicKey, signature, message)(inputs);
		this.print(result);
	}
}

exports.default = VerifyCommand;
VerifyCommand.args = [{
	name: 'publicKey',
	description: 'Public key of the signer of the message.',
	required: true
}, {
	name: 'signature',
	description: 'Signature to verify.',
	required: true
}, {
	name: 'message',
	description: 'Message to verify.'
}];

VerifyCommand.flags = Object.assign({}, _base2.default.flags, {
	message: _command.flags.string(_flags2.default.message)
});

VerifyCommand.description = `
Verifies a signature for a message using the signer’s public key.
`;

VerifyCommand.examples = ['message:verify 647aac1e2df8a5c870499d7ddc82236b1e10936977537a3844a6b05ea33f9ef6 2a3ca127efcf7b2bf62ac8c3b1f5acf6997cab62ba9fde3567d188edcbacbc5dc8177fb88d03a8691ce03348f569b121bca9e7a3c43bf5c056382f35ff843c09 "Hello world"'];