'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _command = require('@oclif/command');

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _flags = require('../../utils/flags');

var _flags2 = _interopRequireDefault(_flags);

var _transfer = require('./create/transfer');

var _transfer2 = _interopRequireDefault(_transfer);

var _secondPassphrase = require('./create/second-passphrase');

var _secondPassphrase2 = _interopRequireDefault(_secondPassphrase);

var _vote = require('./create/vote');

var _vote2 = _interopRequireDefault(_vote);

var _delegate = require('./create/delegate');

var _delegate2 = _interopRequireDefault(_delegate);

var _multisignature = require('./create/multisignature');

var _multisignature2 = _interopRequireDefault(_multisignature);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * LiskHQ/lisk-commander
 * Copyright © 2017–2018 Lisk Foundation
 *
 * See the LICENSE file at the top-level directory of this distribution
 * for licensing information.
 *
 * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
 * no part of this software, including this file, may be copied, modified,
 * propagated, or distributed except according to the terms contained in the
 * LICENSE file.
 *
 * Removal or modification of this copyright notice is prohibited.
 *
 */
const MAX_ARG_NUM = 3;

const typeNumberMap = {
	0: 'transfer',
	1: 'second-passphrase',
	2: 'delegate',
	3: 'vote',
	4: 'multisignature'
};

const options = Object.entries(typeNumberMap).reduce((accumulated, [key, value]) => [...accumulated, key, value], []);

const typeClassMap = {
	transfer: _transfer2.default,
	'second-passphrase': _secondPassphrase2.default,
	vote: _vote2.default,
	delegate: _delegate2.default,
	multisignature: _multisignature2.default
};

const resolveFlags = (accumulated, [key, value]) => {
	if (key === 'type') {
		return accumulated;
	}
	if (typeof value === 'string') {
		return [...accumulated, `--${key}`, value];
	}
	const boolKey = value === false ? `--no-${key}` : `--${key}`;
	return [...accumulated, boolKey];
};

class CreateCommand extends _base2.default {
	async run() {
		const { argv, flags } = this.parse(CreateCommand);
		const { type } = flags;
		const commandType = Object.keys(typeNumberMap).includes(type) ? typeNumberMap[type] : type;
		const resolvedFlags = Object.entries(flags).reduce(resolveFlags, []);
		await typeClassMap[commandType].run([...argv, ...resolvedFlags]);
	}
}

exports.default = CreateCommand;
CreateCommand.flags = Object.assign({}, _base2.default.flags, {
	type: _command.flags.string({
		char: 't',
		description: 'type of transaction to create',
		required: true,
		options
	}),
	passphrase: _command.flags.string(_flags2.default.passphrase),
	'second-passphrase': _command.flags.string(_flags2.default.secondPassphrase),
	'no-signature': _command.flags.boolean(_flags2.default.noSignature),
	votes: _command.flags.string(_flags2.default.votes),
	unvotes: _command.flags.string(_flags2.default.unvotes)
});

CreateCommand.args = new Array(MAX_ARG_NUM).fill().map(i => ({
	name: `${i}_arg`
}));

CreateCommand.description = `
Creates a transaction object.
`;

CreateCommand.examples = ['transaction:create --type=0 100 13356260975429434553L', 'transaction:create --type=delegate lightcurve'];