'use strict';

Object.defineProperty(exports, "__esModule", {
	value: true
});

var _liskElements = require('lisk-elements');

var _liskElements2 = _interopRequireDefault(_liskElements);

var _command = require('@oclif/command');

var _base = require('../../base');

var _base2 = _interopRequireDefault(_base);

var _utils = require('../../utils/input/utils');

var _error = require('../../utils/error');

var _transactions = require('../../utils/transactions');

var _transactions2 = _interopRequireDefault(_transactions);

var _input = require('../../utils/input');

var _input2 = _interopRequireDefault(_input);

var _flags = require('../../utils/flags');

var _flags2 = _interopRequireDefault(_flags);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

/*
 * LiskHQ/lisk-commander
 * Copyright © 2017–2018 Lisk Foundation
 *
 * See the LICENSE file at the top-level directory of this distribution
 * for licensing information.
 *
 * Unless otherwise agreed in a custom licensing agreement with the Lisk Foundation,
 * no part of this software, including this file, may be copied, modified,
 * propagated, or distributed except according to the terms contained in the
 * LICENSE file.
 *
 * Removal or modification of this copyright notice is prohibited.
 *
 */
const getTransactionInput = async () => {
	try {
		const { data } = await (0, _utils.getStdIn)({ dataIsRequired: true });
		if (!data) {
			throw new _error.ValidationError('No transaction was provided.');
		}
		return data;
	} catch (e) {
		throw new _error.ValidationError('No transaction was provided.');
	}
};

class SignCommand extends _base2.default {
	async run() {
		const {
			args: { transaction },
			flags: {
				passphrase: passphraseSource,
				'second-passphrase': secondPassphraseSource
			}
		} = this.parse(SignCommand);

		const transactionInput = transaction || (await getTransactionInput());
		const transactionObject = (0, _transactions2.default)(transactionInput);

		const { passphrase, secondPassphrase } = await (0, _input2.default)({
			passphrase: {
				source: passphraseSource,
				repeatPrompt: true
			},
			secondPassphrase: !secondPassphraseSource ? null : {
				source: secondPassphraseSource,
				repeatPrompt: true
			}
		});

		const result = _liskElements2.default.transaction.utils.prepareTransaction(transactionObject, passphrase, secondPassphrase);

		this.print(result);
	}
}

exports.default = SignCommand;
SignCommand.args = [{
	name: 'transaction',
	description: 'Transaction to sign in JSON format.'
}];

SignCommand.flags = Object.assign({}, _base2.default.flags, {
	passphrase: _command.flags.string(_flags2.default.passphrase),
	'second-passphrase': _command.flags.string(_flags2.default.secondPassphrase)
});

SignCommand.description = `
Sign a transaction using your secret passphrase.
`;

SignCommand.examples = ['transaction:sign \'{"amount":"100","recipientId":"13356260975429434553L","senderPublicKey":null,"timestamp":52871598,"type":0,"fee":"10000000","recipientPublicKey":null,"asset":{}}\''];